import os.path

import pyxel
from pyxel.ui import ColorPicker, NumberPicker, RadioButton

from .constants import EDITOR_IMAGE_X, EDITOR_IMAGE_Y, TEXT_LABEL_COLOR, TOOL_PENCIL
from .drawing_panel import DrawingPanel
from .editor import Editor
from .image_panel import ImagePanel
from .utility import copy_array2d


class ImageEditor(Editor):
    _COLOR_BUTTONS = (
        pyxel.KEY_1,
        pyxel.KEY_2,
        pyxel.KEY_3,
        pyxel.KEY_4,
        pyxel.KEY_5,
        pyxel.KEY_6,
        pyxel.KEY_7,
        pyxel.KEY_8,
    )

    def __init__(self, parent):
        super().__init__(parent)

        self._drawing_panel = DrawingPanel(self, is_tilemap_mode=False)
        self._image_panel = ImagePanel(self, is_tilemap_mode=False)
        self._color_picker = ColorPicker(self, 11, 156, 7, with_shadow=False)
        self._tool_button = RadioButton(
            self,
            81,
            161,
            pyxel.IMAGE_BANK_FOR_SYSTEM,
            EDITOR_IMAGE_X + 63,
            EDITOR_IMAGE_Y,
            7,
            TOOL_PENCIL,
        )
        self._image_picker = NumberPicker(
            self, 192, 161, 0, pyxel.USER_IMAGE_BANK_COUNT - 1, 0
        )

        self.add_event_handler("undo", self.__on_undo)
        self.add_event_handler("redo", self.__on_redo)
        self.add_event_handler("drop", self.__on_drop)
        self.add_event_handler("update", self.__on_update)
        self.add_event_handler("draw", self.__on_draw)
        self._color_picker.add_event_handler(
            "mouse_hover", self.__on_color_picker_mouse_hover
        )
        self.add_tool_button_help(self._tool_button)
        self.add_number_picker_help(self._image_picker)

    @property
    def color(self):
        return self._color_picker.value

    @color.setter
    def color(self, value):
        self._color_picker.value = int(value)

    @property
    def tool(self):
        return self._tool_button.value

    @tool.setter
    def tool(self, value):
        self._tool_button.value = value

    @property
    def image(self):
        return self._image_picker.value

    @property
    def drawing_x(self):
        return self._drawing_panel.viewport_x

    @drawing_x.setter
    def drawing_x(self, value):
        self._drawing_panel.viewport_x = value

    @property
    def drawing_y(self):
        return self._drawing_panel.viewport_y

    @drawing_y.setter
    def drawing_y(self, value):
        self._drawing_panel.viewport_y = value

    def __on_undo(self, data):
        img = data["image"]
        x, y = data["pos"]
        copy_array2d(pyxel.image(img).data, x, y, data["before"])

        self.drawing_x = x
        self.drawing_y = y
        self.parent.image = img

    def __on_redo(self, data):
        img = data["image"]
        x, y = data["pos"]
        copy_array2d(pyxel.image(img).data, x, y, data["after"])

        self.drawing_x = x
        self.drawing_y = y
        self.parent.image = img

    def __on_drop(self, filename):
        _, ext = os.path.splitext(filename)

        if ext.lower() == ".png":
            pyxel.image(self.image).load(0, 0, filename)
            return

    def __on_update(self):
        self.check_tool_button_shortcuts()

        if not pyxel.btn(pyxel.KEY_ALT):
            for btn in self._COLOR_BUTTONS:
                if pyxel.btnp(btn):
                    col = btn - pyxel.KEY_1
                    if pyxel.btn(pyxel.KEY_SHIFT):
                        col += 8
                    self._color_picker.value = col
                    break

    def __on_draw(self):
        self.draw_panel(11, 156, 136, 17)
        self.draw_panel(157, 156, 72, 17)
        pyxel.text(170, 162, "IMAGE", TEXT_LABEL_COLOR)

    def __on_color_picker_mouse_hover(self, x, y):
        self.help_message = "COLOR:1-8/SHIFT+1-8"



import pyxel


def draw_palette(x, y, col):
    col_val = pyxel.DEFAULT_PALETTE[col]
    hex_col = "#{:06X}".format(col_val)
    rgb_col = "{},{},{}".format(col_val >> 16, (col_val >> 8) & 0xFF, col_val & 0xFF)

    pyxel.rect(x, y, 13, 13, col)
    pyxel.text(x + 16, y + 1, hex_col, 7)
    pyxel.text(x + 16, y + 8, rgb_col, 7)
    pyxel.text(x + 5 - (col // 10) * 2, y + 4, "{}".format(col), 7 if col < 6 else 0)

    if col == 0:
        pyxel.rectb(x, y, 13, 13, 13)


pyxel.init(255, 81, caption="Pyxel Color Palette")
pyxel.cls(0)

for i in range(16):
    draw_palette(2 + (i % 4) * 64, 4 + (i // 4) * 20, i)

pyxel.show()



def get_array2d_size(arr):
    if isinstance(arr, list):
        return len(arr[0]), len(arr)
    else:
        return 256, 256


def new_array2d(width, height):
    return [[0] * width for _ in range(height)]


def fill_array2d(arr, val):
    width, height = get_array2d_size(arr)

    for i in range(height):
        for j in range(width):
            arr[i][j] = val


def slice_array2d(arr, x, y, width, height):
    sliced_arr = new_array2d(width, height)

    for i in range(height):
        for j in range(width):
            sliced_arr[i][j] = arr[y + i][x + j]

    return sliced_arr


def copy_array2d(dst, dx, dy, src, sx=0, sy=0, cw=None, ch=None):
    dw, dh = get_array2d_size(dst)
    sw, sh = get_array2d_size(src)

    cw = cw or sw
    ch = ch or sh

    rx1 = max(max(-dx, 0), max(-sx, 0))
    ry1 = max(max(-dy, 0), max(-sy, 0))
    rx2 = max(max(dx + cw - dw, 0), max(sx + cw - sw, 0))
    ry2 = max(max(dy + ch - dh, 0), max(sy + ch - sh, 0))

    cw -= rx1 + rx2
    ch -= ry1 + ry2

    if cw <= 0 or ch <= 0:
        return

    dx += rx1
    dy += ry1
    sx += rx1
    sy += ry1

    for i in range(ch):
        for j in range(cw):
            dst[dy + i][dx + j] = src[sy + i][sx + j]



import pyxel


def draw_triangle(x1, y1, x2, y2, x3, y3, n):
    if n == 0:
        return

    col = n + 7

    if n % 2 == 0:
        pyxel.tri(x1, y1, x2, y2, x3, y3, col)
    else:
        pyxel.trib(x1, y1, x2, y2, x3, y3, col)

    h1 = (x1 + x2) / 2
    w1 = (y1 + y2) / 2
    h2 = (x2 + x3) / 2
    w2 = (y2 + y3) / 2
    h3 = (x3 + x1) / 2
    w3 = (y3 + y1) / 2

    triangle_list.append((x1, y1, h1, w1, h3, w3, n - 1))
    triangle_list.append((h1, w1, x2, y2, h2, w2, n - 1))
    triangle_list.append((h3, w3, h2, w2, x3, y3, n - 1))


triangle_list = [(100, 24, 7, 143, 193, 143, 7)]

pyxel.init(200, 150, caption="Pyxel Triangle API")
pyxel.cls(13)
pyxel.text(6, 6, "tri(x1,y1,x2,y2,x3,y3,col)", 7)
pyxel.text(6, 14, "trib(x1,y1,x2,y2,x3,y3,col)", 7)

while True:
    if triangle_list:
        t = triangle_list.pop(0)
        draw_triangle(*t)

    if pyxel.btnp(pyxel.KEY_Q):
        pyxel.quit()

    pyxel.flip()
